package com.incubator.game.listener;

import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.RequestFilter;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.core.pool.WSRequestPool;
import com.incubator.core.pool.WSResponsePool;
import com.incubator.game.GGame;
import com.incubator.common.net.Connection;
import com.incubator.game.player.GPlayer;
import com.incubator.core.net.handler.listener.GameServerConnectionListener;
import com.incubator.game.player.Player;
import com.incubator.message.proto.CommonProto;
import io.netty.buffer.ByteBuf;

import java.util.HashMap;
import java.util.Map;

/**
 * 公共接口
 * 
 * @author johnc
 *
 */
public class PublicListener extends GameServerConnectionListener {

	@Override
	public void connectionOpened(Connection connection) {
		HeartConfig heart = new HeartConfig();
		connection.setAttribute(HeartConfig, heart);
		setHeartEnable(GGame.heartEnable);
		// 设置最后心跳时间
		connection.setAttribute(LastReadTimeMills, System.currentTimeMillis());
		logger.info("链路打开 : {}", connection);
	}

	@Override
	public void connectionClosed(Connection connection) {
		try {
			GPlayer player = GGame.onlinePlayers.get(connection.getPlayerId());
			if (player != null && player.getSession().getHashCode() == connection.hashCode()) {
				// 执行玩家登出操作
				((Player) player).destroy();
			}
		} catch (Exception e) {
			logger.error("处理玩家退出时发生异常: ", e);
		} finally {
			if (connection != null) {
				connection.close();
				connection = null;
			}
			logger.info("连接关闭: {}", connection);
		}
	}

	@Override
	public void messageArrived(Connection conn, Object msg) {
		if (msg instanceof ByteBuf) {
			ByteBuf byteBuf = (ByteBuf) msg;

			byte[] bytes = null;
			WSRequest request = null;
			WSResponse response = null;

            try {
				// 减少对象分配，通过 ByteBuf 的内置方法获取直接内存访问
				int readableBytes = byteBuf.readableBytes();
				if (readableBytes > 0) {
					bytes = new byte[readableBytes];
					byteBuf.getBytes(byteBuf.readerIndex(), bytes); // 避免修改 ByteBuf 的读取指针
				} else {
					logger.error("收到空消息，断开连接");
					conn.close();
					return;
				}

				// 使用对象池或预先初始化的对象避免频繁创建
				request = WSRequestPool.getRequest(bytes); // 从对象池获取 WSRequest 实例

				// 使用请求过滤器
				if (!RequestFilter.isAllowed(conn, request.getCmd())) {
					logger.warn("请求过于频繁，已过滤: cmd={}, player={}", request.getCmd(), conn.getPlayerId());
					return;
				}

				// 获取处理器
                NetHandler handler = GGame.handlers.get(request.getCmd());
                if (handler == null) {
					logger.error("消息号错误... cmd={}", request.getCmd());
					conn.close();
					return;
                }

				// 心跳消息特殊处理
				if (request.getCmd() == CommonProto.Cmd.HeartBeatReq_VALUE) {
					this.processHeartbeat(conn);
					return;
				}

				// 使用池化的 WSResponse 实例，避免频繁创建
				response = WSResponsePool.getResponse();

				// 处理非心跳消息
				logger.info("收到消息 : cmd={}, 请求内容 : {}", request.getCmd(), request.toJson());
				handler.onDate(conn, request, response);
				conn.writeAndFlush(response.toBytes());
				logger.info("返回消息 : cmd={}, 返回内容 : {}", request.getCmd(), response.toJson());
			} catch (Exception e) {
				logger.error("处理消息时发生错误 : ", e);
				conn.close();
            } finally {
				// 回收池化对象，避免内存泄漏
				if (request != null) WSRequestPool.recycle(request);
				if (response != null) WSResponsePool.recycle(response);
			}
        }
	}

	private void processHeartbeat(Connection conn) throws Exception {
		// 设置最后心跳时间
		conn.setAttribute("LastReadTimeMillis", System.currentTimeMillis());
		// 设置心跳
		resetHeartSeq(conn);
		// 发送心跳响应
		this.doSendHeartRsp(conn);
	}

	@Override
	public boolean doIsHeartReq(Object msg) {
		try {
			ByteBuf byteBuf = (ByteBuf) msg;
			byte[] bytes = new byte[byteBuf.readableBytes()];
			byteBuf.readBytes(bytes);
			WSRequest request = new WSRequest(bytes);
			return request.getCmd() == CommonProto.Cmd.HeartBeatReq_VALUE;
		} catch (Exception e) {
			logger.error("心跳请求解析失败", e);
			return false;
		}
	}

	@Override
	public void doSendHeartReq(Connection conn) {// 发送心跳请求
        try {
            long now = System.currentTimeMillis();
            long lastTime = (long) conn.getAttribute(LastReadTimeMills);
            if (now - lastTime > 10000L) {
//                logger.info("玩家网络断了：" + conn.getUid());
//                conn.close();
            } else {
//                HeartbeatPush push = new HeartbeatPush();
//                push.setTime(System.currentTimeMillis());
//                Message msg = new Message(ProtoCommon.MsgId.S2C_Heartbeat, push.toJson());
//                msg.setCode(ProtoCommon.Code.OK_VALUE);
//                conn.writeAndFlush(super.packMessage(msg));
            }
        } catch (Exception e) {
            logger.info("发送心跳请求出错：{}, {}", conn.getUid(), e.getMessage());
        }

        // {//借助心跳完成一些任务
		// if(connection.getId() != null && connection.getId() > 0) {
		// logger.info("心跳用户：{}" , connection.getId());
		// }
		// }
	}

	@Override
	public void doSendHeartRsp(Connection connection) {
		// 发送心跳回复
        try {
            Map<String, Object> data = new HashMap<>();
            data.put("currentTimeMillis", System.currentTimeMillis());
            connection.writeAndFlush(new WSResponse(CommonProto.Cmd.HeartBeatRes, data).toBytes());
        } catch (Exception e) {
            logger.info("发送心跳回复出错：{}, {}", connection.getUid(), e.getMessage());
        }
    }
}
